        /* Copyright (c) 2007 by IBM Corporation */

dojo.provide("zero.form.widget.field.DateField");
dojo.require("dijit.form.DateTextBox");
dojo.require("zero.form.widget.field.TimeDateFieldMixin");
dojo.require("zero.form.widget.field.properties.DateRangePropertyHandlerMixin");

dojo.declare("zero.form.widget.field.DateField", 
    [dijit.form.DateTextBox, zero.form.widget.field.TimeDateFieldMixin],
    {
        dateType: true,
        lang: "en-us",
		
		startup: function()
		{
			zero.form.widget.field.DateField.superclass.startup.apply(this, arguments);
			
			if (!((undefined == this.defaultValue) || (null == this.defaultValue))) 
				this.textbox.value = this.defaultValue;
		
			this.setValue(this.defaultValue); // cause a validate action
			this.validate();
		},
		
		onfocus: function()
        {
			zero.form.widget.field.TimeDateFieldMixin.prototype.onfocus.apply(this, arguments);
			// summary: open the TimePicker popup
			this._open();
        },

		onkeyup: function(/*Event*/e)
		{
			//zero.form.widget.field.TimeDateFieldMixin.prototype.onkeyup.apply(this, arguments);
        	if (e.charOrCode === undefined)
        	{
        		e.charOrCode = e.keyChar || e.keyCode;
        	}
        	//console.log('opened = ' + this._opened);
        	dijit.form.DateTextBox.prototype._onKeyPress.apply(this, arguments);
		},

		getHandlers: function()
		{
			return [new zero.form.widget.field.properties.DateRangePropertyHandlerMixin()];
		},

        getValue: function()
        {
			return dijit.form.DateTextBox.prototype.getValue.apply(this, arguments);
        },

        setDefaultValue: function(pDefValue)
        {
        	if (typeof(pDefValue) !== 'string')
        	{
        		// convert date value to string
        		pDefValue = this._convertDateToYyyyMmDd(pDefValue);
        	}
        	zero.form.widget.field.DateField.superclass.setDefaultValue.apply(this, arguments);
        },

        setValue: function(pValue)
        {
        	if (typeof(pValue) === 'string')
        	{
        		pValue = this._convertYyyyMmDdToDate(pValue);
        	}
			dijit.form.DateTextBox.prototype.setValue.apply(this, arguments);
        },

		_convertYyyyMmDdToDate: function(pDateStr)
		{
			if (pDateStr !== undefined && pDateStr !== null && pDateStr !== "")
			{
				// parse the date in format yyyy-mm-dd
				var dateParts = pDateStr.split("-");
	        	if (dateParts.length == 3)
	        	{
	            	var result = new Date();
                	result.setFullYear(dateParts[0], dateParts[1] - 1, dateParts[2]);
					result.setHours(0);
					result.setMinutes(0);
					result.setSeconds(0);
					result.setMilliseconds(0);
                	return result;
	        	}
			}

			return "";
		},
		
		_convertDateToYyyyMmDd: function(pDate)
	    {
	        if (pDate  &&  typeof(pDate) != 'string')
	        {
	            var month = '' + (pDate.getMonth() + 1);
	            if (1 == month.length)
	            {
	                month = '0' + month;
	            }
	            var day = '' + pDate.getDate();
	            if (1 == day.length)
	            {
	                day = '0' + day;
	            }
	            return pDate.getFullYear() + '-' + month + '-' + day;
	        }
	        else
	        {
	            return pDate;
	        }
	    },

		getValueForDB: function()
        {
        	var time = this.getValue();
        	return this._convertDateToYyyyMmDd(time);
        },
        
        // this function is overriden from _DateTimeTextBox
    	// The reason we needed to ovveride is that it looks it has a bug and it
    	// constantly passing "date" parameter to the compare(constraints.max, date, "date")
    	// third argument , ignoring the posibility it is a timePicker.
    	
    	// Eventually we decided to allow the user to pick a value which is out of range
    	// so the isDisabledDate function returns hard coded falsae value.
    	// The user will still recieve the red out of range error message.
		_open: function(){
			// summary:
			//	opens the TimePicker, and sets the onValueSelected value
			if(this.disabled || this.readOnly || !this.popupClass){return;}

			var textBox = this;

			if(!this._picker){
				var PopupProto=dojo.getObject(this.popupClass, false);
				this._picker = new PopupProto({
					onValueSelected: function(value){
						textBox.focus(); // focus the textbox before the popup closes to avoid reopening the popup
						setTimeout(dojo.hitch(textBox, "_close"), 1); // allow focus time to take
						// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
						dijit.form._DateTimeTextBox.superclass.setValue.call(textBox, value, true);
					},
					lang: textBox.lang,
					constraints: textBox.constraints,
					isDisabledDate: function(/*Date*/ date){
						// summary:
						// 	disables dates outside of the min/max of the _DateTimeTextBox
						return false;
						/*var compare = dojo.date.compare;
						var constraints = textBox.constraints;
						return constraints && (constraints.min && (compare(constraints.min, date, "date") > 0) || 
							(constraints.max && compare(constraints.max, date, "date") < 0));
						*/
					},
					_onBlur: function()
					{
						textBox._close();
					}
				});
			}
			this._picker.setValue(this.getValue() || new Date());
			this._picker._populateGrid();
			
			if(!this._opened){
				dijit.popup.open({
					parent: this,
					popup: this._picker,
					around: this.domNode,
					onCancel: dojo.hitch(this, this._close),
					onClose: function(){ 
						textBox._opened=false; 
					}
				});
				this._opened=true;
			}
			
			dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth });
			
			dijit.focus(this._picker.domNode);
		},

		_onBlur: function()
		{
		}
    }
);
