        /* Copyright (c) 2007 by IBM Corporation */

dojo.provide("zero.form.widget.field.TimeField");

dojo.require("dijit.form.TimeTextBox");
dojo.require("zero.form.widget.field.TimeDateFieldMixin");
dojo.require("zero.form.widget.field.properties.TimeRangePropertyHandlerMixin");

dojo.declare("zero.form.widget.field.TimeField", 
    [dijit.form.TimeTextBox, zero.form.widget.field.TimeDateFieldMixin], 
    {
        timeType: true,
        
        postMixInProperties: function()
        {
			zero.form.widget.field.TimeField.superclass.postMixInProperties.apply(this, arguments);
			if (!this.constraints) 
			{
				this.constraints = {};
			}
            this.constraints.timePattern = 'HH:mm';
        },

		startup: function()
		{
			zero.form.widget.field.TimeField.superclass.startup.apply(this, arguments);
			
			if (!((undefined == this.defaultValue) || (null == this.defaultValue)))
				this.textbox.value = this.defaultValue;
				
			this.setValue(this.defaultValue); // cause a validate action
			this.validate();
		},
		
		getHandlers: function()
		{
			return [new zero.form.widget.field.properties.TimeRangePropertyHandlerMixin()];
		},

		getValue: function() 
        {    
            return dijit.form.TimeTextBox.prototype.getValue.apply(this, arguments);
        },

        setDefaultValue: function(pDefValue)
        {
        	if (typeof(pDefValue) !== 'string')
        	{
        		// convert date value to string
        		pDefValue = this._convertHhMmToTime(pDefValue);
        	}
        	zero.form.widget.field.TimeField.superclass.setDefaultValue.apply(this, arguments);
        },

		setValue: function(pValue) 
        {    
        	if (typeof(pValue) === 'string')
        	{
        		pValue = this._convertHhMmToTime(pValue);
        	}
            dijit.form.TimeTextBox.prototype.setValue.apply(this, arguments);
        },

		_convertHhMmToTime: function(pTimeStr)
		{
			if (pTimeStr !== undefined && pTimeStr !== null && pTimeStr !== "")
			{
				// parse the time in format hh:mm or hh:mm:ss
				var timeParts = pTimeStr.split(":");
	        	if (timeParts.length == 3 || timeParts.length == 2)
	        	{
	            	var result = new Date();
					result.setTime(0);
					result.setHours(timeParts[0]);
					result.setMinutes(timeParts[1]);
					if (timeParts.length == 3) 
					{
						result.setSeconds(timeParts[2]);
					}
					else
					{
						result.setSeconds(0);
					}
                	return result;
	        	}
			}

			return "";
		},

		getValueForDB: function()
        {
        	var time = this.getValue();
        	var t = new Date(time);
        	var retString = t.getHours()+":"+t.getMinutes()+":"+t.getSeconds();
        	
        	return retString;
        },
	
        
        // this function is overriden from _DateTimeTextBox
    	// The reason we needed to ovveride is that it looks it has a bug and it
    	// constantly passing "date" parameter to the compare(constraints.max, date, "date")
    	// third argument , ignoring the posibility it is a timePicker.
    	
    	// Eventually we decided to allow the user to pick a value which is out of range
    	// so the isDisabledDate function returns hard coded falsae value.
    	// The user will still recieve the red out of range error message. 
		_open: function(){
			// summary:
			//	opens the TimePicker, and sets the onValueSelected value
			if(this.disabled || this.readOnly || !this.popupClass){return;}

			var textBox = this;

			if(!this._picker){
				var PopupProto=dojo.getObject(this.popupClass, false);
				this._picker = new PopupProto({
					onValueSelected: function(value){

						textBox.focus(); // focus the textbox before the popup closes to avoid reopening the popup
						setTimeout(dojo.hitch(textBox, "_close"), 1); // allow focus time to take

						// this will cause InlineEditBox and other handlers to do stuff so make sure it's last
						dijit.form._DateTimeTextBox.superclass.setValue.call(textBox, value, true);
					},
					lang: textBox.lang,
					constraints: textBox.constraints,
					isDisabledDate: function(/*Date*/ date){
						// summary:
						// 	disables dates outside of the min/max of the _DateTimeTextBox
						return false;
						/*var compare = dojo.date.compare;
						var constraints = textBox.constraints;
						return constraints && (constraints.min && (compare(constraints.min, date, "time") > 0) || 
							(constraints.max && compare(constraints.max, date, "time") < 0));
						*/
					},
					_onBlur: function()
					{
						textBox._close();
					}
				});
				this._picker.setValue(this.getValue() || new Date());
			}
			if(!this._opened){
				dijit.popup.open({
					parent: this,
					popup: this._picker,
					around: this.domNode,
					onCancel: dojo.hitch(this, this._close),
					onClose: function(){ textBox._opened=false; }
				});
				this._opened=true;
			}
			
			dojo.marginBox(this._picker.domNode,{ w:this.domNode.offsetWidth });
		}
    }
);
